'use strict';

var path = require('path');
var resolve = require('resolve');

var _require = require('../../utils'),
    hasPkgProp = _require.hasPkgProp,
    hasFile = _require.hasFile;

var useBuiltinConfig = !hasFile('.lintstagedrc') && !hasFile('lint-staged.config.js') && !hasPkgProp('lint-staged');

var lintStagedPath = require.resolve('lint-staged');
var cosmiconfigPath = resolve.sync('cosmiconfig', {
  basedir: path.dirname(lintStagedPath)
});

// lint-staged uses cosmiconfig to find its configuration
// and it has no other way to provide config
// (via a node API or command-line flag)
// So, we're doing this require cache magic to provide our own
// config so folks don't have to have that in their package.json
function fakeCosmiconfig() {
  if ((arguments.length <= 0 ? undefined : arguments[0]) === 'lint-staged') {
    return Promise.resolve({ config: require('../../config/lintstagedrc') });
  } else {
    return require(cosmiconfigPath).apply(undefined, arguments);
  }
}

if (useBuiltinConfig) {
  require.cache[cosmiconfigPath] = { exports: fakeCosmiconfig };
}

require(lintStagedPath);