'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var fs = require('fs');
var path = require('path');
var arrify = require('arrify');
var has = require('lodash.has');
var readPkgUp = require('read-pkg-up');
var which = require('which');

var _readPkgUp$sync = readPkgUp.sync({
  cwd: fs.realpathSync(process.cwd())
}),
    pkg = _readPkgUp$sync.pkg,
    pkgPath = _readPkgUp$sync.path;

var appDirectory = path.dirname(pkgPath);

function resolveKcdScripts() {
  if (pkg.name === 'kcd-scripts') {
    return require.resolve('./').replace(process.cwd(), '.');
  }
  return resolveBin('kcd-scripts');
}

// eslint-disable-next-line complexity
function resolveBin(modName) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$executable = _ref.executable,
      executable = _ref$executable === undefined ? modName : _ref$executable,
      _ref$cwd = _ref.cwd,
      cwd = _ref$cwd === undefined ? process.cwd() : _ref$cwd;

  var pathFromWhich = void 0;
  try {
    pathFromWhich = fs.realpathSync(which.sync(executable));
  } catch (_error) {
    // ignore _error
  }
  try {
    var modPkgPath = require.resolve(`${modName}/package.json`);
    var modPkgDir = path.dirname(modPkgPath);

    var _require = require(modPkgPath),
        bin = _require.bin;

    var binPath = typeof bin === 'string' ? bin : bin[executable];
    var fullPathToBin = path.join(modPkgDir, binPath);
    if (fullPathToBin === pathFromWhich) {
      return executable;
    }
    return fullPathToBin.replace(cwd, '.');
  } catch (error) {
    if (pathFromWhich) {
      return executable;
    }
    throw error;
  }
}

var fromRoot = function () {
  for (var _len = arguments.length, p = Array(_len), _key = 0; _key < _len; _key++) {
    p[_key] = arguments[_key];
  }

  return path.join.apply(path, [appDirectory].concat(p));
};
var hasFile = function () {
  return fs.existsSync(fromRoot.apply(undefined, arguments));
};
var ifFile = function (files, t, f) {
  return arrify(files).some(function (file) {
    return hasFile(file);
  }) ? t : f;
};

var hasPkgProp = function (props) {
  return arrify(props).some(function (prop) {
    return has(pkg, prop);
  });
};

var hasPkgSubProp = function (pkgProp) {
  return function (props) {
    return hasPkgProp(arrify(props).map(function (p) {
      return `${pkgProp}.${p}`;
    }));
  };
};

var ifPkgSubProp = function (pkgProp) {
  return function (props, t, f) {
    return hasPkgSubProp(pkgProp)(props) ? t : f;
  };
};

var hasScript = hasPkgSubProp('scripts');
var hasPeerDep = hasPkgSubProp('peerDependencies');
var hasDep = hasPkgSubProp('dependencies');
var hasDevDep = hasPkgSubProp('devDependencies');
var hasAnyDep = function () {
  for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  return [hasDep, hasDevDep, hasPeerDep].some(function (fn) {
    return fn.apply(undefined, args);
  });
};

var ifPeerDep = ifPkgSubProp('peerDependencies');
var ifDep = ifPkgSubProp('dependencies');
var ifDevDep = ifPkgSubProp('devDependencies');
var ifAnyDep = function (deps, t, f) {
  return hasAnyDep(deps) ? t : f;
};
var ifScript = ifPkgSubProp('scripts');

function parseEnv(name, def) {
  if (process.env.hasOwnProperty(name)) {
    return JSON.parse(process.env[name]);
  }
  return def;
}

function getConcurrentlyArgs(scripts) {
  var colors = ['bgBlue', 'bgGreen', 'bgMagenta', 'bgCyan', 'bgWhite', 'bgRed', 'bgBlack', 'bgYellow'];
  var prefixColors = Object.keys(scripts).reduce(function (pColors, _s, i) {
    return pColors.concat([`${colors[i % colors.length]}.bold.reset`]);
  }, []).join(',');

  // prettier-ignore
  return ['--kill-others-on-fail', '--prefix', '[{name}]', '--names', Object.keys(scripts).join(','), '--prefix-colors', prefixColors].concat(_toConsumableArray(Object.values(scripts).map(function (s) {
    return JSON.stringify(s);
  })));
}

module.exports = {
  ifDevDep,
  ifPeerDep,
  ifScript,
  ifDep,
  ifAnyDep,
  hasPkgProp,
  appDirectory,
  fromRoot,
  hasScript,
  resolveBin,
  resolveKcdScripts,
  parseEnv,
  pkg,
  hasFile,
  ifFile,
  getConcurrentlyArgs
};